/*===========================================================================

FILE: CXTApp.c
===========================================================================*/


/*===============================================================================
INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"          // Module interface definitions
#include "AEEAppGen.h"          // Applet interface definitions
#include "AEEShell.h"           // Shell interface definitions
#include "AEEFile.h"			// File interface definitions
#include "AEE.h"
#include "AEEStdLib.h"

#include "CXTWasRun.h"
#include "CXTApp.bid"

#define TEST_BUF_SIZE 50

/*-------------------------------------------------------------------
Applet structure. All variables in here are reference via "pMe->"
-------------------------------------------------------------------*/
// create an applet structure that's passed around. All variables in
// here will be able to be referenced as static.
typedef struct _CXTApp {
	AEEApplet      a ;	       // First element of this structure must be AEEApplet
    AEEDeviceInfo  DeviceInfo; // always have access to the hardware device information
    IDisplay      *pIDisplay;  // give a standard way to access the Display interface
    IShell        *pIShell;    // give a standard way to access the Shell interface

    // add your own variables here...
      AEEDeviceInfo  m_di;          // always have access to the hardware device information
      int            m_cLineNum;    // screen line to be written to next
    AEERect        dispRect;			//testing screen

} CXTApp;

/*-------------------------------------------------------------------
Function Prototypes
-------------------------------------------------------------------*/
static  boolean CXTApp_HandleEvent(CXTApp* pMe, AEEEvent eCode, 
                                             uint16 wParam, uint32 dwParam);
boolean CXTApp_InitAppData(CXTApp* pMe);
void    CXTApp_FreeAppData(CXTApp* pMe);
void    CXTApp_DisplayOutput(IApplet * pi, int nline, char *pszStr, AEEFont font);
void    CXTApp_WriteLine(CXTApp * pMe, char *pszStr);
void    CXTApp_testRunning(CXTApp* pMe);
/*===============================================================================
FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================
FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IDisplay_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   IDisplay_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell object. 

	pIModule: pin]: Contains pointer to the IModule object to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId, IShell *pIShell, IModule *po, void **ppObj)
{
	*ppObj = NULL;

	if( ClsId == AEECLSID_CXTAPP )
	{
		// Create the applet and make room for the applet structure
		if( AEEApplet_New(sizeof(CXTApp),
                          ClsId,
                          pIShell,
                          po,
                          (IApplet**)ppObj,
                          (AEEHANDLER)CXTApp_HandleEvent,
                          (PFNFREEAPPDATA)CXTApp_FreeAppData) ) // the FreeAppData function is called after sending EVT_APP_STOP to the HandleEvent function
                          
		{
			//Initialize applet data, this is called before sending EVT_APP_START
            // to the HandleEvent function
			if(CXTApp_InitAppData((CXTApp*)*ppObj))
			{
				//Data initialized successfully
				return(AEE_SUCCESS);
			}
			else
			{
				//Release the applet. This will free the memory allocated for the applet when
				// AEEApplet_New was called.
				IAPPLET_Release((IApplet*)*ppObj);
				return EFAILED;
			}

        } // end AEEApplet_New

    }

	return(EFAILED);
}


/*===========================================================================
FUNCTION SampleAppWizard_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean SampleAppWizard_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean CXTApp_HandleEvent(CXTApp* pMe, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  

    switch (eCode) 
	{
        // App is told it is starting up
        case EVT_APP_START:                        
		    // Add your code here...
            CXTApp_testRunning(pMe);

            return(TRUE);


        // App is told it is exiting
        case EVT_APP_STOP:
            // Add your code here...

      		return(TRUE);


        // App is being suspended 
        case EVT_APP_SUSPEND:
		    // Add your code here...

      		return(TRUE);


        // App is being resumed
        case EVT_APP_RESUME:
		    // Add your code here...

      		return(TRUE);


        // An SMS message has arrived for this app. Message is in the dwParam above as (char *)
        // sender simply uses this format "//BREW:ClassId:Message", example //BREW:0x00000001:Hello World
        case EVT_APP_MESSAGE:
		    // Add your code here...

      		return(TRUE);

        // A key was pressed. Look at the wParam above to see which key was pressed. The key
        // codes are in AEEVCodes.h. Example "AVK_1" means that the "1" key was pressed.
        case EVT_KEY:
		    // Add your code here...

      		return(TRUE);


        // If nothing fits up to this point then we'll just break out
        default:
            break;
   }

   return FALSE;
}


// this function is called when your application is starting up
boolean CXTApp_InitAppData(CXTApp* pMe)
{
      int charHeight;

    // Get the device information for this handset.
    // Reference all the data by looking at the pMe->DeviceInfo structure
    // Check the API reference guide for all the handy device info you can get
    pMe->m_di.wStructSize = sizeof(pMe->m_di);
    ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_di);

    // The display and shell interfaces are always created by
    // default, so we'll asign them so that you can access
    // them via the standard "pMe->" without the "a."
    pMe->pIDisplay = pMe->a.m_pIDisplay;
    pMe->pIShell   = pMe->a.m_pIShell;

    // Insert your code here for initializing or allocating resources...

  charHeight = IDISPLAY_GetFontMetrics(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, NULL, NULL);

  //if it is toucan display 176*220, with 40 phone infor bar
  if(pMe->m_di.cxScreen ==176 && pMe->m_di.cyScreen >= 180)
  {
    pMe->dispRect.x = 10;
    pMe->dispRect.y = charHeight*3 + 5;
    pMe->dispRect.dx = pMe->m_di.cxScreen-20;
    pMe->dispRect.dy = pMe->m_di.cyScreen - charHeight*5 - 10;
  }
  //else display size are either 100*100 or very samll one
  else{
    //if screen size is bigger than 100*(2*charHeight + 100) 
    //[one charHeight for progress bar, one for display name]
    //set display screen to 100 * 100
    //else set to device screen size
    if(pMe->m_di.cxScreen > 100)
      pMe->dispRect.x = (pMe->m_di.cxScreen - 100)/2;
    else
      pMe->dispRect.x = 0;
	   
    if (pMe->m_di.cyScreen > (2*charHeight + 100))
    {
      pMe->dispRect.y = pMe->m_di.cyScreen - charHeight*2 - 100;
      pMe->dispRect.dy = 100;
      pMe->dispRect.dx = 100;
    }
    else
    {
      pMe->dispRect.y  = charHeight*2;
      pMe->dispRect.dy = pMe->m_di.cyScreen - pMe->dispRect.y;
      pMe->dispRect.dx = pMe->m_di.cxScreen - pMe->dispRect.x;
    }
  }


    // if there have been no failures up to this point then return success
    return TRUE;
}

// this function is called when your application is exiting
void CXTApp_FreeAppData(CXTApp* pMe)
{
    // insert your code here for freeing any resources you have allocated...

    // example to use for releasing each interface:
    // if ( pMe->pIMenuCtl != NULL )         // check for NULL first
    // {
    //    IMENUCTL_Release(pMe->pIMenuCtl)   // release the interface
    //    pMe->pIMenuCtl = NULL;             // set to NULL so no problems trying to free later
    // }
    //

}


/*===========================================================================

FUNCTION: CXTApp_DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the
    resulting value in pixels is set to the y-coordinate of the start of
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed
    string.

    How many characters that fit on one line is calculated for each line
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters
          the output might differ on different handsets (skins). Where some
          handsets will have a smaller screen and large default fonts which will
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).

PROTOTYPE:
   void CXTApp_DisplayOutput(IApplet * pi, int nline, char *pszStr, AEEFont font)

PARAMETERS:
   pi:     [in]: Contains a pointer to the IApplet class.
   nline:  [in]: Contains the line number to start displaying the text.
        The line numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr: [in]: The character string to be displayed on the screen.

DEPENDENCIES
   None

RETURN VALUE
   None

SIDE EFFECTS
   None

===========================================================================*/
void
CXTApp_DisplayOutput(IApplet * pi, int nline, char *pszStr, AEEFont font)
{
  AEEDeviceInfo di;           // Device Info
  AECHAR szBuf[100] = { 0 };  // a buffer that supports 200 char string
  AECHAR *psz = NULL;
  AEERect rc;
  int pixelWidth;
  int pnFits = 0, dy;
  int totalCh = 0;
  int charHeight = 0;         // Stores the char height in pixels for given font
  int pnAscent = 0;           // Stores the ascent in number of pixels
  int pnDescent = 0;          // Stores the descent in number of pixels
  AEEApplet *pMe = (AEEApplet *) pi;
  if (pMe == NULL)
    return;

  // Get device information
  ISHELL_GetDeviceInfo (pMe->m_pIShell, &di);

  // Get the font metrics info
  charHeight = IDISPLAY_GetFontMetrics (pMe->m_pIDisplay, font, &pnAscent, &pnDescent);

  // Convert to wide string (unicode)
  STR_TO_WSTR ((char *) pszStr, szBuf, sizeof (szBuf));

  // If nlines is zero then print this string starting around the middle of
  // the screen. Or else multiply nlines by 10 to decide the y coordinate of
  // the start of the string.
  if (nline < 0)
  {
    dy = di.cyScreen * 2 / 5;
  }
  else
  {
    dy = nline * charHeight;
  }

  // psz keeps track of the point from which to write from the string buffer
  // in case the string does not fit one line and needs to wrap around in the
  // next line.
  psz = szBuf;

  // Need to calculate the lotal string length to decide if any wrapping
  // around is needed.
  totalCh = STRLEN ((char *) pszStr);

  // Keep displaying text string on multiple lines if the string can't be displayed
  // on one single line. Lines are spaced 15 pixels apart.
  while ((totalCh > 0) && (*psz != NULL))
  {
    // Get information on how many characters will fit in a line.
    // Give the pointer to the buffer to be displayed, and the number of
    // pixels along the x axis you want to display the string in (max number)
    // pnFits will have the max number of chars that will fit in the maxWidth
    // number of pixels (given string can't fit in one line), or the number of
    // chars in the string (if it does fit in one line). pnWidth gives the
    // number of pixels that will be used to display pnFits number of chars.
    pixelWidth = IDISPLAY_MeasureTextEx (pMe->m_pIDisplay, font, (AECHAR *) psz,
                                         -1, di.cxScreen - 5,
                                         &pnFits);  // Number of chars that will fit a line

    // If pnFits is zero there is something wrong in the input to above function.
    // Normally this scenario should not occur. But, have the check anyway.
    if (pnFits == 0)
      return;

    SETAEERECT(&rc, 5, dy, di.cxScreen, charHeight);
    IDISPLAY_EraseRect(pMe->m_pIDisplay, &rc);
    IDISPLAY_DrawText (pMe->m_pIDisplay, font, psz, pnFits,
                       5 /*start dx */ ,
                       dy, 0 /* use default rectangle coordinates */ ,
                       IDF_ALIGN_CENTER);
    psz += pnFits;          // move pointer to the next segment to be displayed
    totalCh -= pnFits;      // reduce the total number of characters to still display
    dy += charHeight;       // Place next line charHeight pixels below the
    // previous line.
    IDISPLAY_Update (pMe->m_pIDisplay);
    if (totalCh < pnFits)
      pnFits = totalCh;   // if total number is less than pnFits, adjust pnFits
  }
  return;
}

/*===========================================================================
FUNCTION: CXTApp_WriteLine

DESCRIPTION
    This function displays a lines of text on the screen, taking into account
    the number of text lines printed so far and the number of lines available
    for output.

    If all available lines are exhausted, this function returns without doing
    anything.

    If the last available line of the screen is reached, this function prints
    "..." to indicate that some lines may not have been printed due to lack
    of space.

    Otherwise, this function prints the line on the screen by calling DisplayOutput,
    and updates the count of lines printed based on how many lines DisplayOutput
    used to print the text.


PROTOTYPE:
    void CXTApp_WriteLine(CXTApp * pMe,  char *pszStr)

PARAMETERS:
    pMe:   [in]: Contains a pointer to this app's data structure.
    pszStr:    [in]: The character string to be displayed on the screen.

DEPENDENCIES
    Assumes the pMe->m_cLineNum gets initialized to the starting line for text display
    on the screen, and that pMe->m_cMaxLine contains the last available line for
    output on the screen.

RETURN VALUE
    None.

SIDE EFFECTS
    None

===========================================================================*/
void CXTApp_WriteLine(CXTApp * pMe, char *pszStr)
{
    int charHeight;

    charHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL, NULL, NULL);
    pMe->m_cLineNum++;

	if(charHeight == 0)
		return;
    if ((pMe->m_di.cyScreen / charHeight) < pMe->m_cLineNum)
       pMe->m_cLineNum = 0;

    CXTApp_DisplayOutput ((IApplet *) pMe, pMe->m_cLineNum, pszStr, AEE_FONT_NORMAL);
    return;
}

void CXTApp_testRunning(CXTApp* pMe)
{    
    int wasRun;
    char pBuf[TEST_BUF_SIZE];
    IWasRun theWasRun;

    IWasRun* pTest = WasRun_Ctor(&theWasRun);

    wasRun = IWASRUN_wasRun(pTest);
    SPRINTF(pBuf, "wasRun:%d", wasRun);
    CXTApp_WriteLine(pMe, pBuf);

    IWASRUN_testMethod(pTest);

    wasRun = IWASRUN_wasRun(pTest);
    SPRINTF(pBuf, "wasRun:%d", wasRun);
    CXTApp_WriteLine(pMe, pBuf);
}
