#include "AEE.h"

typedef enum 
{
    APPLET_EXIT = (EVT_USER + 0x700),
    APPLET_EXIT_MAX
} AppletExitEvent;

//#define AppletExitEvent uint16
//#define APPLET_EXIT (EVT_USER + 0x100)

struct IExitAppletView
{
    void            (*AddExitAppletEvent)(AppletExitEvent evt);
    AppletExitEvent (*GetExitAppletEvent)(void);
    void            (*SetExitAppletStatus)(boolean bSuccess);
};

struct IExitAppletModel
{
    boolean (*ExitApplet)(AppletExitEvent evt);
};

class ExitApplectViewMock {
public:

    void            AddExitAppletEvent(AppletExitEvent evt)
    {
        SubmittedExitEvent = evt;
    }

    AppletExitEvent GetExitAppletEvent(void)
    {
        return SubmittedExitEvent;
    }

    void SetExitApplectStatus(boolean bSuccess)
    {
        bExitAppletStatus = bSuccess;
    }

    void FireExitAppletEvent(void)
    {
    }

public:
    boolean bExitAppletStatus;

private:
    AppletExitEvent SubmittedExitEvent;
};

class ExitApplectModelMock {
public:
    boolean exitSuccess;
    AppletExitEvent exitInputEvent;
public:
    boolean ExitApplet(AppletExitEvent evt)
    {
        exitInputEvent = evt;        
        return exitSuccess;
    }
};

#if defined(MAKE_ASSERTIONS)
#include <assert.h>
#define ASSERT(x) assert(x)
#endif 


void test_ExitApplet_Pass(void)
{
    ExitApplectModelMock    modelMock;
    ExitApplectViewMock     viewMock;
    
    // force the exit process to succeed 
    modelMock.exitSuccess = TRUE;
    
    // simulates the user input by firing an event
    // that the presenter should handle
    viewMock.FireExitAppletEvent();

#if defined(MAKE_ASSERTIONS)

    // Verify that what the user entered as the event code is 
    // what the model actually uses as input.
    ASSERT(APPLET_EXIT == modelMock.exitInputEvent);    

    // Verify that the view has been updated to reflect a successful exit
    ASSERT(viewMock.bExitAppletStatus == TRUE);

#endif // defined(MAKE_ASSERTIONS)
}

void test_ExitApplet_Fail(void)
{
    ExitApplectModelMock    modelMock;
    ExitApplectViewMock     viewMock;
    
    // force the exit process to fail
    modelMock.exitSuccess = FALSE;
    
    // simulates the user input by firing an event
    // that the presenter should handle
    viewMock.FireExitAppletEvent();

#if defined(MAKE_ASSERTIONS)

    // Verify that what the user entered as the event code is 
    // what the model actually uses as input.
    ASSERT(APPLET_EXIT == modelMock.exitInputEvent);

    // Verify that the view has been updated to reflect an unsuccessful exit
    ASSERT(viewMock.bExitAppletStatus == FALSE);    
    
#endif // defined(MAKE_ASSERTIONS)
}