// MyTestApp.cpp: implementation of the cxTestApp class.
//
//////////////////////////////////////////////////////////////////////

/*===============================================================================
INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"          // Module interface definitions
#include "AEEAppGen.h"          // Applet interface definitions
#include "AEEShell.h"           // Shell interface definitions
#include "AEEFile.h"			// File interface definitions
#include "AEERootForm.h"

#include "testcxtestapp.h"
#include "AppForm.h"

#include "testcxtest.bid"

#define TESTCXTEST_THEME_FILE "testcxtest_theme.bar"

extern "C"
{

/*===========================================================================
FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IDisplay_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   IDisplay_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell object. 

	pIModule: pin]: Contains pointer to the IModule object to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId, IShell *pIShell, IModule *po, void **ppObj)
{
	*ppObj = NULL;

	if( ClsId == AEECLSID_MYTESTAPP )
	{
		// Create the applet and make room for the applet structure
		if( AEEApplet_New(sizeof(cxTestApp),
                          ClsId,
                          pIShell,
                          po,
                          (IApplet**)ppObj,
                          (AEEHANDLER)cxTestApp::HandleEvent,
                          (PFNFREEAPPDATA)cxTestApp::freeAppData) ) // the FreeAppData function is called after sending EVT_APP_STOP to the HandleEvent function
                          
		{
			//Initialize applet data, this is called before sending EVT_APP_START
            // to the HandleEvent function
            if(cxTestApp::initAppData((IApplet*)*ppObj))
			{
				//Data initialized successfully
				return(AEE_SUCCESS);
			}
			else
			{
				//Release the applet. This will free the memory allocated for the applet when
				// AEEApplet_New was called.
				IAPPLET_Release((IApplet*)*ppObj);
				return EFAILED;
			}

        } // end AEEApplet_New

    }

	return(EFAILED);
}

}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//cxTestApp::cxTestApp()
//{
//
//}
//
//cxTestApp::~cxTestApp()
//{
//
//}

/*===========================================================================
FUNCTION cxTestApp::HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean SampleAppWizard_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
boolean cxTestApp::HandleEvent(cxTestApp *pApp, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
    return pApp->onEvent(eCode, wParam, dwParam);
}

// this function is called when your application is starting up
boolean cxTestApp::initAppData(IApplet *pIApplet)
{
    return ((cxTestApp *) pIApplet)->initData();
}

// this function is called when your application is exiting
void cxTestApp::freeAppData(cxTestApp *pApp)
{
	pApp->freeData();
}


/* Initialize applet data*/
boolean cxTestApp::initData()
{
    int result;

    // Get the device information for this handset.
    // Reference all the data by looking at the pMe->DeviceInfo structure
    // Check the API reference guide for all the handy device info you can get
    DeviceInfo.wStructSize = sizeof(DeviceInfo);
    ISHELL_GetDeviceInfo(m_pIShell,&DeviceInfo);

    // The display and shell interfaces are always created by
    // default, so we'll asign them so that you can access
    // them via the standard "pMe->" without the "a."
    pIDisplay = m_pIDisplay;
    pIShell   = m_pIShell;

    // Insert your code here for initializing or allocating resources...
   
    mainForm = NULL;
    
    //create rootForm
    result = ISHELL_CreateInstance(pIShell, AEECLSID_ROOTFORM, (void**) &rootForm);
    
    //set the theme
    if(result == 0)
        result = IROOTFORM_SetThemeFileName(rootForm, TESTCXTEST_THEME_FILE);
    
    
    // if there have been no failures up to this point then return success
    if (result == 0)
       return TRUE;

    return FALSE;
}

// Free app data
void cxTestApp::freeData()
{
    // insert your code here for freeing any resources you have allocated...
    
    if(rootForm) {
        IROOTFORM_Release(rootForm);
        rootForm = NULL;
    }
    
    if(mainForm) {
        IFORM_Release(mainForm);
        mainForm = NULL;
    }
    
    // example to use for releasing each interface:
    // if ( pMe->pIMenuCtl != NULL )         // check for NULL first
    // {
    //    IMENUCTL_Release(pMe->pIMenuCtl)   // release the interface
    //    pMe->pIMenuCtl = NULL;             // set to NULL so no problems trying to free later
    // }
    //
}

extern void test_ExitApplet_Fail(void);
extern void test_ExitApplet_Pass(void);

boolean cxTestApp::onEvent(AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
    // Allow rootform to handle event first
    if(IROOTFORM_HandleEvent(rootForm, eCode, wParam, dwParam)) {
        return TRUE;
    }

    switch (eCode) 
    {
        // App is told it is starting up
    case EVT_APP_START:          
        {
            // create main app form and push it onto the stack
            if(AppForm_New(&mainForm, pIShell, rootForm, pIDisplay) == 0) {
                IROOTFORM_PushForm(rootForm, mainForm);
                return TRUE;
            }
            break;
            
           
            
            return(TRUE);
        }
        
        // App is told it is exiting
    case EVT_APP_STOP:
        // Add your code here...
        
      		return(TRUE);
            
            
            // App is being suspended 
    case EVT_APP_SUSPEND:
        // Add your code here...
        
      		return(TRUE);
            
            
            // App is being resumed
    case EVT_APP_RESUME:
        // Add your code here...
        
      		return(TRUE);
            
            
            // An SMS message has arrived for this app. Message is in the dwParam above as (char *)
            // sender simply uses this format "//BREW:ClassId:Message", example //BREW:0x00000001:Hello World
    case EVT_APP_MESSAGE:
        // Add your code here...
        
      		return(TRUE);
            
            // A key was pressed. Look at the wParam above to see which key was pressed. The key
            // codes are in AEEVCodes.h. Example "AVK_1" means that the "1" key was pressed.
    case EVT_KEY:// Handle necessary key events
        {
            switch(wParam) 
            {
            case AVK_CLR: // Handle actions when the user presses the clear key
                {
                    return TRUE;
                }
                break;
                
            case AVK_LEFT: // Handle the left button 
                {
                    //test_ExitApplet_Pass();
                    return TRUE;
                }
                break;
                
            case AVK_RIGHT:
                {
                    //test_ExitApplet_Fail();
                    return TRUE;
                }
                break;
                
            case AVK_UP:
                {
                    return TRUE;
                }
                break;
                
            case AVK_DOWN:
                {
                    return TRUE;
                }
                break;
                    
            case AVK_1:  // If a numeric key is pressed, set the number of pixels the pen should
            case AVK_2:  // move each time one of the direction keys is pressed.
            case AVK_3:
            case AVK_4:
            case AVK_5:
            case AVK_6:
            case AVK_7:
            case AVK_8:
            case AVK_9:
            case AVK_0:
                {
                    return TRUE;
                }
                break;
                        
            case AVK_SELECT:  // Handle select key events when we are in drawing mode.
                {
                }
                break;
            }            
        }
        break;
        
        // If nothing fits up to this point then we'll just break out
    default:
        break;
   }
   
   return FALSE;
   
}

