/* Simple error message routines for QW */

#ifndef _QW_error_h
#define _QW_error_h

#include "begin_code.h"
/* Set up for C function definitions, even when using C++ */
#ifdef __cplusplus
extern "C" {
#endif

/* Public functions */
extern DECLSPEC void QWCALL QW_SetError(const char *fmt, ...);
extern DECLSPEC char * QWCALL QW_GetError(void);
extern DECLSPEC void QWCALL QW_ClearError(void);

/* Private error message function - used internally */
#define QW_OutOfMemory()	QW_Error(QW_ENOMEM)
typedef enum {
	QW_ENOMEM,
	QW_EFREAD,
	QW_EFWRITE,
	QW_EFSEEK,
	QW_LASTERROR
} QW_errorcode;
extern void QW_Error(QW_errorcode code);

#ifdef __SD_DEBUG__ 

#   define __sd_str(n) #n
#   define __sd_location(n) __FUNCTION__"() at " __FILE__":" __sd_str(n)
#   define sd_location  __sd_location(__LINE__)

#   define sd_debug(a_format, ...) \
	(getenv("SD_DEBUG") ? fprintf(stderr, "[DEBUG] "sd_location" "a_format"\n", ##__VA_ARGS__ ) : 0)
#   define sd_error(a_format, ...) \
	(getenv("SD_ERROR") ? fprintf(stderr, "[ERROR] "sd_location" "a_format"\n", ##__VA_ARGS__ ) : 0)
#   define sd_oserror(afunc, aparam) \
        (sd_error("%s(%s): #%d %s", afunc, aparam, errno, strerror(errno)), -1)

#else

#   define sd_debug
#   define sd_error
#   define sd_oserror

#endif

/* backward compatibility */
#define os_error sd_oserror


/* Ends C function definitions when using C++ */
#ifdef __cplusplus
}
#endif
#include "close_code.h"

#endif /* _QW_error_h */
