#ifndef _QW_thread_h
#define _QW_thread_h

/* Header for the QW thread management routines 

	These are independent of the other QW routines.
*/

#include "QW_main.h"
#include "QW_types.h"

/* Thread synchronization primitives */
#include "QW_mutex.h"

#include "begin_code.h"
/* Set up for C function definitions, even when using C++ */
#ifdef __cplusplus
extern "C" {
#endif

/* The QW thread structure, defined in QW_thread.c */
struct QW_Thread;
typedef struct QW_Thread QW_Thread;

/* Create a thread */
extern DECLSPEC QW_Thread * QWCALL QW_CreateThread(int (*fn)(void *), void *data);

/* Get the 32-bit thread identifier for the current thread */
extern DECLSPEC Uint32 QWCALL QW_ThreadID(void);

/* Get the 32-bit thread identifier for the specified thread,
   equivalent to QW_ThreadID() if the specified thread is NULL.
 */
extern DECLSPEC Uint32 QWCALL QW_GetThreadID(QW_Thread *thread);

/* Wait for a thread to finish.
   The return code for the thread function is placed in the area
   pointed to by 'status', if 'status' is not NULL.
 */
extern DECLSPEC void QWCALL QW_WaitThread(QW_Thread *thread, int *status);

/* Forcefully kill a thread without worrying about its state */
extern DECLSPEC void QWCALL QW_KillThread(QW_Thread *thread);


/* Ends C function definitions when using C++ */
#ifdef __cplusplus
}
#endif
#include "close_code.h"

#endif /* _QW_thread_h */
